package org.w3c.rdf.util;

import org.w3c.rdf.model.*;
import org.w3c.rdf.vocabulary.rdf_syntax_19990222.RDF;
import org.w3c.rdf.util.RDFUtil;
import org.w3c.tools.sorter.*;
import java.util.*;

/**
 * This class provides support for experimental ordering in RDF
 */

public class OrderUtil implements Comparer {

  static final OrderUtil sorter = new OrderUtil();

  public int compare(Object handle, Object o1, Object o2) {

    try {
      Model m = (Model)handle;
      Statement t1 = (Statement)o1;
      Statement t2 = (Statement)o2;

      //      int res = t1.predicate().getURI().compareTo( t2.predicate().getURI() );
      
      //      if(res != 0)
      //	return res;

      // same predicates, compare objects
      try {
	String orderStr1 = RDFUtil.getObjectLiteral(m, t1, RDF.order);
	String orderStr2 = RDFUtil.getObjectLiteral(m, t2, RDF.order);

	if(orderStr1 != null && orderStr2 != null) {
	  int order1 = Integer.parseInt(orderStr1);
	  int order2 = Integer.parseInt(orderStr2);
	  if(order1 < order2)
	    return -1;
	  else if(order1 > order2)
	    return 1;
	  else
	    return 0;
	}
      } catch (NumberFormatException nfe) {
      } catch (NullPointerException npe) {
      }

    } catch (ModelException me) {}
    
    return 0; // do nothing
  }


  public static Vector getOrderedStatementVector(Model m) throws ModelException {

    Vector tv = RDFUtil.getStatementVector(m);
    QuickSort.sort(tv, sorter, m);
    return tv;
  }

  public static void main(String[] args) throws Exception {

    // testing closure
    if(args.length == 0) {
      System.err.println("OrderUtil <instance URL>");
      System.exit(1);
    }

    RDFFactory f = new RDFFactoryImpl();
    Model m = f.createModel();
    RDFUtil.parse(args[0], f.createParser(), m);

    System.out.println("Ordered statements:");
    Vector v = getOrderedStatementVector(m);
    for(int i = 0; i < v.size(); i++) {
      Statement s = (Statement)v.elementAt(i);
      System.out.println(s.toString());
    }
  }
}
